<?php 

namespace Teknisa\CNC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;

use Teknisa\CNC\Services;
use Teknisa\Libs\Util\Environment;

class Cnc02600ContCorEstabelecimento extends \Teknisa\CNC\Controllers\TeknisaCrud {
	protected $service;
	protected $environment;
	
	public function __construct (Services\Cnc02600ContCorEstabelecimentoService $service, Environment $environment) {
        $this->service = $service;
        $this->environment = $environment;
	}

	public function getEstabelecParam(DTO\Request $request, DTO\Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            
            $resultadoFiltrado = $this->service->getEstabelecParam($params);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc02600_estabelec', $resultadoFiltrado));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
	
	public function saveEstabelec (DTO\Request $request, DTO\Response $response) {
	    try {
    	    $rows = $request->getDataSet()->getRows();
    	    
    	    foreach ($rows as $row) {
    	        if (is_object($row))
                    $row = (array) $row;
                
                $row["NRORG"] = $this->environment->getNrOrgTrab();
                $row["IDATIVO"] = "S";
                $row["DTULTATU"] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                $row["CDOPERULTATU"] = $this->environment->getCdOperador();
                        
                $this->service->saveEstabelec($row);
                $response->addNotification(new Notification("Dados salvos com sucesso!"), Notification::TYPE_SUCCESS);
    	    }
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}
	
	public function deleteEstabelecContcor (DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            
            foreach ($rows as $row) {
                $row["NRORG"] = $this->environment->getNrOrgTrab();
                $this->service->deleteEstabelecContcor($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getAcquirersAut(DTO\Request $request, DTO\Response $response){
        try {
            $row = $request->getFilterCriteria()->getConditions();
            $row["LABELADQUIRENTE"] = isset($row[0]["value"]) ? $row[0]["value"][0]["_ALL"] : "%%";
            $Produto = $this->service->getAcquirersAut($row);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc02600_adquirentesAut', $Produto));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
	
	public function getAdquirenteParam(DTO\Request $request, DTO\Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $resultadoFiltrado = $this->service->getAdquirenteParam($params);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc02600_adquirentes', $resultadoFiltrado));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
	
	public function saveAdquirente (DTO\Request $request, DTO\Response $response) {
	    try {
    	    $rows = $request->getDataSet()->getRows();
    	    
    	    foreach ($rows as $row) {
    	        if (is_object($row))
                    $row = (array) $row;
                
                $row["NRORG"] = $this->environment->getNrOrgTrab();
                $row["IDATIVO"] = "S";
                $row["DTULTATU"] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                $row["CDOPERULTATU"] = $this->environment->getCdOperador();
                        
                $this->service->saveAdquirente($row);
                $response->addNotification(new Notification("Dados salvos com sucesso!"), Notification::TYPE_SUCCESS);
    	    }
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}
	
	public function getProductsAut(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $row = $request->getFilterCriteria()->getConditions();
            $row["LABELPRODUTOADQ"] = isset($row[0]["value"]) ? $row[0]["value"] : "%%";
            $Produto = $this->service->getProductsAut($row);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc02600_produto', $Produto));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
	
	public function getProdAdqParam(DTO\Request $request, DTO\Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $resultadoFiltrado = $this->service->getProdAdqParam($params);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc02600_adquirentes', $resultadoFiltrado));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
	
	public function saveProdutoAdq (DTO\Request $request, DTO\Response $response) {
	    try {
    	    $rows = $request->getDataSet()->getRows();
    	    foreach ($rows as $row) {
    	        if (is_object($row))
                    $row = (array) $row;
                
                $row["NRORG"] = $this->environment->getNrOrgTrab();
                $row["IDATIVO"] = "S";
                $row["DTULTATU"] = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
                $row["CDOPERULTATU"] = $this->environment->getCdOperador();
                        
                $this->service->saveProdutoAdq($row);
                $response->addNotification(new Notification("Dados salvos com sucesso!"), Notification::TYPE_SUCCESS);
    	    }
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}

	public function getAgencias (DTO\Request\Filter $request, DTO\Response $response) {
		try {
			$row = $request->getFilterCriteria()->getConditions();
			$dataSet = $this->service->getAgencias($row);
			$response->addDataSet(new DataSet('dataSet', $dataSet));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}
	
	public function getContas (DTO\Request\Filter $request, DTO\Response $response) {
		try {
			$row = $request->getFilterCriteria()->getConditions();
			$dataSet = $this->service->getContas($row);
			$response->addDataSet(new DataSet('dataSet', $dataSet));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

	public function getBancoMasks (DTO\Request\Filter $request, DTO\Response $response) {
	    try {
	        $row = $request->getFilterCriteria()->getConditions();
	        $dataSet = $this->movFinanceiraService->getBancoMasks($row[0]['value']);
	        $response->addDataSet(new DataSet('dataSet', $dataSet));
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}

	public function getEmpresaFilial (DTO\Request\Filter $request, DTO\Response $response) {
	    try {
	        $row = $request->getFilterCriteria()->getConditions();
	        $dataSet = $this->service->getEmpresaFilial($row[0]['value']);
	        $response->addDataSet(new DataSet('dataSet', $dataSet));
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}

    public function cadastroAutomatico(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $establishments = $row['NRESTABELEC'];
            $purchasers = $row['NRADQUIRENTE'];
            $products = $row['NRPRODUTOADQ'];
            $rowToSave = [
                'CDBANCO' => $row['CDBANCO'],
                'CDAGENCIA' => $row['CDAGENCIA'],
                'CDCONTCORR' => $row['CDCONTCORR'],
                'CDREDUCONT' => $row['CDREDUCONT'],
                'NRORG' => $this->environment->getNrOrgTrab(),
                'IDATIVO' => 'S',
                'DTULTATU' => $this->environment->getCurrentTime()->format("d/m/Y H:i:s"),
                'CDOPERULTATU' => $this->environment->getCdOperador()
            ];
            $purchaserFilter = [
                'name' => 'NRESTABELEC',
                'operator' => '=',
                'value' => ''
            ];
            $loadAllPurchasers = empty($purchasers) && !empty($products);
            foreach($establishments as $establishment) {
                $rowToSave['NRESTABELEC'] = $establishment;
                $this->service->saveEstabelec($rowToSave);
                if($loadAllPurchasers) {
                    $purchaserFilter['value'] = $establishment;
                    $purchasers = array_column($this->service->getAdquirenteParam([$purchaserFilter]), 'NRADQUIRENTE');
                }
                foreach($purchasers as $purchaser) {
                    $rowToSave['NRADQUIRENTE'] = $purchaser;
                    if(!$loadAllPurchasers) {
                        $this->service->saveAdquirente($rowToSave);
                    }
                    foreach($products as $product) {
                        $rowToSave['NRPRODUTOADQ'] = $product;
                        $this->service->saveProdutoAdq($rowToSave);
                    }
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}