<?php 

namespace Teknisa\CNC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

use Teknisa\CNC\Services\Cnc03100DesconciliacaoVendasService;
use Teknisa\Libs\Util\Environment;

use Teknisa\CNC\Utilities\UtilCNC;

class Cnc03100DesconciliacaoVendas {
    protected $entityManager;
    protected $dataSourceName = 'cnc03100_vendas_conciliadas';
    protected $parameterBagColumns;
    protected $conciliacaoManualService;
    protected $environment;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, ParameterBag $parameterBag, Cnc03100DesconciliacaoVendasService $desconciliacaoVendasService, UtilCNC $util, Environment $environment) {
        $this->parameterBag = $parameterBag;
		$this->entityManager = $entityManager;
        $this->desconciliacaoVendasService = $desconciliacaoVendasService;
        $this->util = $util;
        $this->environment = $environment;
    }
    
    public function findVendas(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $params["NRORG"]           = $this->util->getParamValueFromFilter($filter, "NRORG");
            $params["NRESTABELEC"]     = $this->util->getParamValueFromFilter($filter, "NMESTABELEC");
            $params["DTVENDA"]         = $this->util->getParamValueFromFilter($filter, "DTPERIODO");
            $params["CDNSU"]           = $this->util->getParamValueFromFilter($filter, "NSU");
            $params["NRPRODUTOADQ"]    = $this->util->getParamValueFromFilter($filter, "NRPRODUTOADQ");
            $params["IDVISPRODUTOADQ"] = $this->util->getParamValueFromFilter($filter, "IDVISPRODUTOADQ");
            
            $params["NRESTABELEC"] = is_array($params["NRESTABELEC"]) && !empty($params["NRESTABELEC"]) ? 
                    $this->util->buildParamValue($params["NRESTABELEC"]) : null;
            $params["NRPRODUTOADQ"] = is_array($params["NRPRODUTOADQ"]) && !empty($params["NRPRODUTOADQ"]) ? 
                $this->util->buildParamValue($params["NRPRODUTOADQ"]) : null;
            
            $params["TNRESTABELEC"]  = $params["NRESTABELEC"] ? null : "T";      
            $params["TNRPRODUTOADQ"] = $params["NRPRODUTOADQ"] ? null : "T";
            $params["TCDNSU"]        = $params["CDNSU"] ? null : "T";
            $params["CDNSU"]         = $params["TCDNSU"] ? null : $params["CDNSU"];
            
            $dataSets = $this->desconciliacaoVendasService->selectConciliacoes($params);
            $response->addDataSet(new DataSet('cnc03100_vendas_conciliadas', $dataSets[0]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verificaBoletoBaixadoCnc(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $rows = $request->getRow();
            $bool = true;
            
            $boletoLancado = self::verifyContReceb($rows);
            if($boletoLancado > 0){
                $verificaFinanceiro["BOLETO"] = true;
            }else{
                $verificaFinanceiro["BOLETO"] = false;
            }
            
            $response->setStatus($bool);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('DesconciliacaoFinanceiro', $verificaFinanceiro));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function verificaConciliacoesSimultaneas(DTO\Request\Row $request, DTO\Response $resposta){
        try{
            $connection = $this->entityManager->getConnection();
            $rows = $request->getRow();
            $bool = true;
            
            foreach($rows[0] as $row){
                $simultaneas = $this->desconciliacaoVendasService->verificaConciliacoesSimultaneas($row);

                if($simultaneas['NRCONCLANC'] > 1){
                    $verificaSimultaneas["EXIST"] = true;
                } else {
                    $verificaSimultaneas["EXIST"] = false;
                }
            }
            
            $resposta->setStatus($bool);
            $resposta->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('VerificaSimultaneas', $verificaSimultaneas));
        } catch (\Exception $e) {
            return $e->getMessage();
        }
    }
    
    public function verifyContReceb($rows){
        try{
            foreach($rows as $row){
                $contreceb = $this->desconciliacaoVendasService->verifyContReceb($row)["FLAGBOLETO"];
                if($contreceb){
                    return $contreceb;
                }
            }
            return $contreceb;
        } catch (\Exception $e) {
            return $e->getMessage();
        }
    }
    
    public function desconciliacaoVendas (DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows = $request->getRow();
            $bool = true;
            
            //Deleta informações da ITEMCORVENDA
            foreach ($rows[0] as $row) {
                $NRCONSMOVCAIXA = $row["CHAVE"];
                $DTVENDA = $row["DTVENDA"];
                $NRORG = $this->environment->getNrOrgTrab();
                
                if (isset($row['NRCONCLANC'])) {
                    //Salva as datas da venda para realizar a Replicação ao final do processo
                    $datasVendas = $this->desconciliacaoVendasService->getDatasVendas($row);
                    
                    //Deleta os registros relacionados à conciliação da ITEMCORVENDA
                    $this->desconciliacaoVendasService->deleteItemCorVenda($NRCONSMOVCAIXA);
                    
                    //Retira a conciliação da CONSMOVCAIXA, RETVENDAADQ, RETFINACADQ e deleta as informaçãoes da DIVCONCILIACAO e CONICILIACAOLANC
                    $bool = $bool && $this->desconciliacaoVendasService->desconciliacaoVendas($row);    
                }
                
                if (isset($row['NRLANCTONF'])) {
                    //Apaga as informações da venda da CONTARECEBER e da BOLETRECEBER
                    $bool = $bool && $this->desconciliacaoVendasService->deletaFinanceiro($row);
                }
                
                if (isset($row['CHAVE'])) {
                    //Deleta a CONSMOVCAIXA
                    $bool = $bool && $this->desconciliacaoVendasService->deletaConsmovcaixa($row);
                }
                
                //Executa a Replicação de Vendas para preencher a CONSMOVCAIXA com as novas informações
                foreach ($datasVendas as $data) {
                    $this->desconciliacaoVendasService->executaReplicacao($data["DTVENDA"]);    
                }
            }
            
            $response->setStatus($bool);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}